@@ -19,7 +19,8 @@
  * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  * 
- * $NCDId: @(#)Alibint.c,v 1.24 1994/08/12 17:17:04 greg Exp $
+ * $Id: Alibint.c,v 1.5 2000/12/13 04:22:37 jon Exp $
+ * $NCDId: @(#)Alibint.c,v 1.28 1995/12/28 19:42:47 greg Exp $
  */
 
 /* Portions derived from */
@@ -52,13 +53,16 @@ without express or implied warranty.
 #include <audio/Aos.h>
 #include "Alibnet.h"
 #include <stdio.h>
+#ifndef WIN32
 #ifdef SYSV	/* fd_set */
 #include <sys/socket.h>
 #endif
+#endif /* !WIN32 */
 
 /* check for both EAGAIN and EWOULDBLOCK, because some supposedly POSIX
  * systems are broken and return EWOULDBLOCK when they should return EAGAIN
  */
+#ifndef WIN32
 #if defined(EAGAIN) && defined(EWOULDBLOCK)
 #define ETEST(err) (err == EAGAIN || err == EWOULDBLOCK)
 #else
@@ -68,6 +72,9 @@ without express or implied warranty.
 #define ETEST(err) (err == EWOULDBLOCK)
 #endif
 #endif
+#else /* WIN32 */
+#define ETEST(err) (WSAGetLastError() == WSAEWOULDBLOCK)
+#endif /* WIN32 */
 
 #ifdef LACHMAN
 #ifdef EMSGSIZE
@@ -122,6 +129,14 @@ static void _AuEnq(
 #endif
 );
 
+static void _AuEventEnqueued(
+#if NeedFunctionPrototypes
+    AuServer *,			/* server */
+    int,			/* who */
+    AuEvent *			/* event */
+#endif
+);
+
 /*
  * The following routines are internal routines used by audiolib for protocol
  * packet transmission and reception.
@@ -159,6 +174,7 @@ _AuWaitForWritable(aud)
     AuServer *aud;
 {
 #if !defined(AMOEBA) && !defined(_MINIX)
+#ifndef WIN32
     AuInt32 r_mask[MSKCNT];
     AuInt32 w_mask[MSKCNT];
     int nfound;
@@ -178,6 +194,28 @@ _AuWaitForWritable(aud)
 	} while (nfound <= 0);
 
 	if (_AuANYSET(r_mask)) {
+#else /* WIN32 */
+    fd_set rset, wset;
+    int nfound;
+
+    while (1)
+    {
+	do
+	{
+	    FD_ZERO(&rset);
+	    FD_ZERO(&wset);
+	    FD_SET(aud->fd, &rset);
+	    FD_SET(aud->fd, &wset);
+	
+	    nfound = select(1, &rset, &wset, NULL, NULL);
+
+	    if (nfound < 0 && WSAGetLastError() != WSAEINTR)
+		_AuIOError(aud);
+	} while (nfound <= 0);
+
+    if (FD_ISSET(aud->fd, &rset))
+    {
+#endif /* WIN32 */
 	    _AuAlignedBuffer buf;
 	    int pend;
 	    register int len;
@@ -217,8 +255,13 @@ _AuWaitForWritable(aud)
 		}
 	    } ENDITERATE
 	}
+#ifndef WIN32
 	if (_AuANYSET(w_mask))
 	    return;
+#else /* WIN32 */
+	if (FD_ISSET(aud->fd, &wset))
+	    return;
+#endif /* WIN32 */
     }
 #else /* AMOEBA || _MINIX */
     /* Its a fatal error when this is called under Amoeba or Minix */
@@ -232,6 +275,7 @@ static void
 _AuWaitForReadable(aud)
   AuServer *aud;
 {
+#ifndef WIN32
 #if !defined(AMOEBA) && !defined(_MINIX)
     AuUint32 r_mask[MSKCNT];
     int result;
@@ -248,6 +292,21 @@ _AuWaitForReadable(aud)
     printf(stderr, "audiolib: _AuWaitForReadable called\n");
     abort();
 #endif /* AMOEBA || _MINIX */
+#else /* WIN32 */
+    while (1)
+    {
+	fd_set set;
+
+	FD_ZERO(&set);
+	FD_SET(aud->fd, &set);
+
+	if (select(1, &set, NULL, NULL, NULL) == 1)
+	    return;
+
+	if (WSAGetLastError() != WSAEINTR)
+	    _AuIOError(aud);
+    }
+#endif /* WIN32 */
 }
 
 
@@ -460,34 +519,37 @@ _AuRead (aud, data, size)
 
 	if ((aud->flags & AuServerFlagsIOError) || size == 0) return;
 	errno = 0;
+
 	while ((bytes_read = ReadFromServer(aud->fd, data, (int)size))
-		!= size) {
-
-	    	if (bytes_read > 0) {
-		    size -= bytes_read;
-		    data += bytes_read;
-		    }
-		else if (ETEST(errno)) {
-		    _AuWaitForReadable(aud);
-		    errno = 0;
-		}
+	       != size) 
+	  {
+     
+	    if (bytes_read > 0) {
+	      size -= bytes_read;
+	      data += bytes_read;
+	    }
+	    else if (ETEST(errno)) {
+	      _AuWaitForReadable(aud);
+	      errno = 0;
+	    }
 #ifdef SUNSYSV
-		else if (errno == 0) {
-		    _AuWaitForReadable(aud);
-		}
+	    else if (errno == 0) {
+	      _AuWaitForReadable(aud);
+	    }
 #endif
-		else if (bytes_read == 0) {
-		    /* Read failed because of end of file! */
-		    errno = EPIPE;
-		    _AuIOError(aud);
-		    }
+	    else if (bytes_read == 0) {
+	      /* Read failed because of end of file! */
+	      errno = EPIPE;
+	      _AuIOError(aud);
+	    }
+	    
+	    else  /* bytes_read is less than 0; presumably -1 */ {
+	      /* If it's a system call interrupt, it's not an error. */
+	      if (errno != EINTR)
+		_AuIOError(aud);
+	    }
 
-		else  /* bytes_read is less than 0; presumably -1 */ {
-		    /* If it's a system call interrupt, it's not an error. */
-		    if (errno != EINTR)
-		    	_AuIOError(aud);
-		    }
-	    	 }
+	  }
 }
 
 #ifdef WORD64
@@ -624,6 +686,7 @@ _AuReadPad (aud, data, size)
     	register AuInt32 bytes_read;
 	struct iovec iov[2];
 	char pad[3];
+	char *p;
 
 	if ((aud->flags & AuServerFlagsIOError) || size == 0) return;
 	iov[0].iov_len = (int)size;
@@ -638,39 +701,57 @@ _AuReadPad (aud, data, size)
 	iov[1].iov_base = pad;
 	size += iov[1].iov_len;
 	errno = 0;
-	while ((bytes_read = ReadvFromServer (aud->fd, iov, 2)) != size) {
+				/* JET - we'll use a little indirect ptr */
+				/* arithmetic to get around the */
+				/* fact that iov_base is often a void * */
 
-	    if (bytes_read > 0) {
+	while ((bytes_read = ReadvFromServer (aud->fd, iov, 2)) != size) 
+	  {
+	    
+	    if (bytes_read > 0) 
+	      {
 		size -= bytes_read;
-	    	if ((iov[0].iov_len -= bytes_read) < 0) {
+	    	if ((iov[0].iov_len -= bytes_read) < 0) 
+		  {
 		    iov[1].iov_len += iov[0].iov_len;
-		    iov[1].iov_base -= iov[0].iov_len;
+
+		    p = (char *)iov[1].iov_base;
+		    p -= iov[0].iov_len;
+		    iov[1].iov_base = p;
+
 		    iov[0].iov_len = 0;
-		    }
+		  }
 	    	else
-	    	    iov[0].iov_base += bytes_read;
-	    	}
-	    else if (ETEST(errno)) {
+		  {
+		    p = (char *)iov[0].iov_base;
+		    p += bytes_read;
+		    iov[0].iov_base = p;
+		  }
+	      }
+	    else if (ETEST(errno)) 
+	      {
 		_AuWaitForReadable(aud);
 		errno = 0;
-	    }
+	      }
 #ifdef SUNSYSV
-	    else if (errno == 0) {
+	    else if (errno == 0) 
+	      {
 		_AuWaitForReadable(aud);
-	    }
+	      }
 #endif
-	    else if (bytes_read == 0) {
+	    else if (bytes_read == 0) 
+	      {
 		/* Read failed because of end of file! */
 		errno = EPIPE;
 		_AuIOError(aud);
-		}
-	    
-	    else  /* bytes_read is less than 0; presumably -1 */ {
+	      }
+	    else  /* bytes_read is less than 0; presumably -1 */ 
+	      {
 		/* If it's a system call interrupt, it's not an error. */
 		if (errno != EINTR)
-		    _AuIOError(aud);
-		}
-	    }
+		  _AuIOError(aud);
+	      }
+	  }
 }
 
 /*
@@ -1156,7 +1237,7 @@ _AuEnq (aud, event, who)
     
 	    aud->tail = qelt;
 	    aud->qlen++;
-	    _AuEventEnqueued(aud, who, qelt->event);
+	    _AuEventEnqueued(aud, who, &qelt->event);
 	} else {
 	    /* ignored, or stashed away for many-to-one compression */
 	    qelt->next = aud->qfree;
@@ -1266,9 +1347,11 @@ register auEvent *event;			/* wire protocol event */
 static char *_SysErrorMsg (n)
     int n;
 {
+#if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__)
     extern char *sys_errlist[];
+#endif
     extern int sys_nerr;
-    char *s = ((n >= 0 && n < sys_nerr) ? sys_errlist[n] : "unknown error");
+    char *s = (char *)((n >= 0 && n < sys_nerr) ? sys_errlist[n] : "unknown error");
 
     return (s ? s : "no such error");
 }
@@ -1280,7 +1363,7 @@ static char *_SysErrorMsg (n)
  * _AuDefaultIOError - Default fatal system error reporting routine.  Called 
  * when an Au internal system error is encountered.
  */
-static void
+void
 _AuDefaultIOError (aud)
 	AuServer *aud;
 {
@@ -1772,12 +1855,14 @@ int iovcnt;
 
 #endif /* CRAY */
 
-#if defined(SYSV) && defined(SYSV386) && !defined(STREAMSCONN)
+#if defined(SYSV) && defined(SYSV386) && !defined(STREAMSCONN) || defined(WIN32)
 #define HAS_FAKE_IOV
 /*
  * SYSV/386 does not have readv so we emulate
  */
+#ifndef WIN32
 #include <sys/uio.h>
+#endif /* !WIN32 */
 
 int _AuReadV (fd, iov, iovcnt)
 int fd;
@@ -1793,7 +1878,11 @@ int iovcnt;
 	base = iov->iov_base;
 	while (len > 0) {
 	    register int nbytes;
+#ifndef WIN32
 	    nbytes = read(fd, base, len);
+#else /* WIN32 */
+	    nbytes = recv(fd, base, len, 0);
+#endif /* WIN32 */
 	    if (nbytes < 0 && total == 0)  return -1;
 	    if (nbytes <= 0)  return total;
 	    errno = 0;
@@ -1805,9 +1894,38 @@ int iovcnt;
     return total;
 }
 
+#ifdef WIN32
+int _AuWriteV (fd, iov, iovcnt)
+int fd;
+struct iovec *iov;
+int iovcnt;
+{
+    int i, len, total;
+    char *base;
+
+    errno = 0;
+    for (i=0, total=0;  i<iovcnt;  i++, iov++) {
+	len = iov->iov_len;
+	base = iov->iov_base;
+	while (len > 0) {
+	    register int nbytes;
+	    nbytes = send(fd, base, len, 0);
+	    if (nbytes < 0 && total == 0)  return -1;
+	    if (nbytes <= 0)  return total;
+	    errno = 0;
+	    len   -= nbytes;
+	    total += nbytes;
+	    base  += nbytes;
+	}
+    }
+    return total;
+}
+#endif /* WIN32 */
+
 #endif /* SYSV && SYSV386 && !STREAMSCONN */
 
-#ifdef STREAMSCONN
+#if defined(STREAMSCONN) 
+
 #define HAS_FAKE_IOV
 /*
  * Copyright 1988, 1989 AT&T, Inc.
@@ -1847,8 +1965,7 @@ int iovcnt;
 	END USER STAMP AREA
 */
 
-
-extern char _AusTypeofStream[];
+extern char _AusTypeOfStream[];
 extern Austream _AusStream[];
 
 #define MAX_WORKAREA 4096
