@@ -19,7 +19,7 @@
  * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  * 
- * $NCDId: @(#)ConnSvr.c,v 1.20 1995/11/28 00:00:40 greg Exp $
+ * $NCDId: @(#)ConnSvr.c,v 1.22 1995/12/06 19:25:17 greg Exp $
  */
 
 /* Portions derived from */
@@ -49,21 +49,31 @@
  * This file contains operating system dependencies.
  */
 
-#ifdef __FreeBSD__
+#ifdef WIN32
+#define ioctl		ioctlsocket
+#define FIOSNBIO	FIONBIO
+#endif /* WIN32 */
+
+#if defined(__FreeBSD__) || defined(__NetBSD__)
 #include <sys/param.h>
 #endif /* __FreeBSD__ */
 #include <ctype.h>
+#include <X11/Xauth.h>			/* to get at the .Xauthority */
 #include <audio/Alibint.h>
 #include <audio/Aos.h>
 #include "Alibnet.h"
-#include <X11/Xauth.h>			/* to get at the .Xauthority */
 #include <stdio.h>
 #ifdef DNETCONN
 #include <netdnet/dn.h>
 #include <netdnet/dnetdb.h>
 #endif
 #ifdef SVR4
-#include <sys/stropts.h>
+# if defined(USL)
+#  include <sys/socket.h>
+#  include <netdb.h>
+#  include <netinet/in.h>
+# endif
+# include <sys/stropts.h>
 #endif
 
 #ifdef STREAMSCONN
@@ -202,7 +212,8 @@ _AuServerWait(sig)
 }
 
 static int
-_AuStartServer()
+_AuStartServer(iserver)
+	int iserver;
 {
 	pid_t pid;
 
@@ -231,16 +242,20 @@ _AuStartServer()
 		_exit(_AuServerUp == AuTrue? 0: 255);
 		/*NOTREACHED*/
 	}
+	else {
+	  char arg1[80];	/* hmmm. */
 
+	  sprintf(arg1,":%d",iserver);
 	/* grandchild -- start server */
 	(void)signal(SIGUSR1, SIG_IGN);
 	/* start server.  Ideally the command should be set by some resource */
-	(void)execlp("au", "au", "-timeout",  "600", (char *)NULL);
+	(void)execlp("nasd", "nasd", arg1, "-pn", "-timeout",  "600", (char *)NULL);
 	perror("exec");
 	_AuServerUp = -1;
-	kill(SIGUSR1, getppid());	/* wake up parent */
+       kill(getppid(), SIGUSR1);       /* wake up parent */
 	_exit(255);
 	/*NOTREACHED*/
+	}
 }
 #endif /* STARTSERVER */
 
@@ -512,7 +527,7 @@ int _AuConnectServer (server_name, fullnamep, svrnump,
 #else /* STARTSERVER */
     {
 	/* if local connection, try to start up a server */
-	if (!_AuIsLocal(phostname) || !_AuIsAudioOK() || !_AuStartServer())
+	if (!_AuIsLocal(phostname) || !_AuIsAudioOK() || !_AuStartServer(iserver))
 	    goto bad;
 	/* try again */
 	iserver = saviserver;
@@ -521,8 +536,10 @@ int _AuConnectServer (server_name, fullnamep, svrnump,
 	    goto bad;
     }
 #endif /* STARTSERVER */
+#ifndef WIN32
     if (fd >= OPEN_MAX)
 	goto bad;
+#endif /* !WIN32 */
 
     /*
      * Set close-on-exec so that programs that fork() doesn't get confused.
@@ -543,11 +560,13 @@ int _AuConnectServer (server_name, fullnamep, svrnump,
 #endif /* FIOSNBIO */
 #endif /* !AMOEBA && !_MINIX */
 
+#ifndef WIN32
 #ifdef FD_CLOEXEC
     (void) fcntl (fd, F_SETFD, FD_CLOEXEC);
 #else
     (void) fcntl (fd, F_SETFD, 1);
 #endif
+#endif /* !WIN32 */
 
     /*
      * Build the expanded server name:
@@ -624,9 +643,11 @@ int _AuConnectServer (server_name, fullnamep, svrnump,
 #ifdef UNIXCONN
 #define NEED_BSDISH
 #endif
+#ifndef WIN32
 #ifdef TCPCONN
 #define NEED_BSDISH
 #endif
+#endif /* !WIN32 */
 #endif /* !_MINIX */
 
 #ifdef NEED_BSDISH			/* makedepend can't handle #if */
@@ -824,6 +845,19 @@ static int MakeTCPConnection (phostname, iserverp, xname, retries,
     int fd;				/* file descriptor to return */
     int len;				/* length tmp variable */
 
+#ifdef WIN32
+    static int winsockInited;
+
+    if (!winsockInited)
+    {
+	WSADATA data;
+
+	if (WSAStartup(0x0101, &data) == -1)
+	    return -1;
+	winsockInited = 1;
+    }
+#endif /* WIN32 */
+
 #define INVALID_INETADDR ((AuUint32) -1)
 
     if (!phostname) {
@@ -1491,7 +1525,11 @@ int _AuDisconnectServer (server)
       vc_close(chandesc->virtcirc, VC_BOTH);
     XAmFreeChanDesc(chandesc);
 #else
+#ifndef WIN32
     (void) close(server);
+#else /* WIN32 */
+    (void) closesocket(server);
+#endif /* WIN32 */
 #endif
     return 0;
 }
@@ -1541,7 +1579,7 @@ _AuSendClientPrefix (aud, client, auth_proto, auth_string)
     len -= MNX_WritevToServer (aud, iovarray, niov);
 #endif
 
-#if !defined(AMOEBA) && !defined(_MINIX)
+#if !defined(AMOEBA) && !defined(_MINIX) && !defined(WIN32)
     /*
      * Set the connection non-blocking since we use select() to block.
      */
